const db = require("../infra/database/db");
const Funcionario = require("../infra/repository/funcionario.repository");
const funcionario = new Funcionario();
const jwt = require("jsonwebtoken");

class funcionarioController {

  login() {
    return function (req, res) {
      const data = req.body;


      funcionario
        .login(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.log(error);
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }

  register() {
    return function (req, res) {
        const data = req.body;


        funcionario.register(data).then((response) => {
            if (response.error) return res.status(response.code).send({ error: response.error });
            else return res.status(200).json(response);
        })
        .catch((error) => {
            console.log(error);
            return res.status(400).send({ error: `Não foi possível realizar o cadastro!` });
        });
    };
  }

  listAllFuncs() {
    return function (req, res) {

      funcionario
        .listAllFuncs()
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  recover() {
    return function (req, res) {
      const  data  = req.body;

    funcionario.recover(data).then((response) => {
        if (response.error)return res.status(response.code).send({ error: response.error });
        else return res.status(200).json(response);
    })
    .catch((error) => {
        console.log(error);
        return res.status(400).send({ error: `Não foi possível realizar o recuperação!` });
    });
    };
  }


  logout() {
    return function (req, res) {
      const auth = req.query.auth ? req.query.auth : "";
      funcionario
        .logout(auth)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }

 
  listAllUser() {
    return function (req, res) {
      const { id } = req.params;

      funcionario
        .listAllUser(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

   
  listFuncionarioById() {
    return function (req, res) {
      const id_func  = req.query.id_func;

      funcionario
        .listFuncionarioById(id_func)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listPedidosPendentes() {
    return function (req, res) {
        const { id } = req.params;
        const id_func = req.query.id_func;

        funcionario.listPedidosPendentes(id, id_func).then((response) => {
            if (response.error) res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
        }).catch((error) => {
            throw new Error(error);
        });
    }
}

registerPedido() {
  return function (req, res) {
      const data = req.body;


      funcionario.registerPedido(data).then((response) => {
          if (response.error) return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
      })
      .catch((error) => {
          console.log(error);
          return res.status(400).send({ error: `Não foi possível realizar o cadastro!` });
      });
  };
}


  listProdutoFunc() {
    return function (req, res) {
        const { id } = req.params;
        const id_func = req.query.id_func;

        funcionario.listProdutoFunc(id, id_func).then((response) => {
            if (response.error) res.status(response.code).send({ error: response.error });
            else return res.status(200).send(response);
        }).catch((error) => {
            throw new Error(error);
        });
    }
  }


 
  /**
   * Delete um usuário
   * @returns mensagem referente ao resultado obtido
   */
  delete() {
    return function (req, res) {
      const { id } = req.params;

      funcionario
        .delete(id)
        .then((response) => {
          if (response.error) res.status(response.code).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  update() {
    return function (req, res) {
      const  data  = req.body;

      funcionario
        .update(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

      /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
    updateQuantidade() {
      return function (req, res) {
          const  data  = req.body;
          
          funcionario.updateQuantidade(data).then((response) => {
              if (response.error) res.status(400).send(response);
              else return res.status(200).send(response);
          }).catch((error) => {
              throw new Error(error);
          });
      }
    }

           /**
     * Delete um usuário
     * @returns mensagem referente ao resultado obtido
     */
      deleteProdutoDoPedido() {
      return function (req, res) {
          const id_produto  = req.query.id_produto;
          const id_func = req.query.id_func;

          
          funcionario.deleteProdutoDoPedido(id_produto, id_func).then((response) => {
              if (response.error) res.status(response.code).send(response);
              else return res.status(200).send(response);
          }).catch((error) => {
              throw new Error(error);
          });
      }
    }


    
    listTabelaDePrecoFunc() {
      return function (req, res) {
          const id_func = req.query.id_func;


          funcionario.listTabelaDePrecoFunc(id_func).then((response) => {
              if (response.error) res.status(response.code).send({ error: response.error });
              else return res.status(200).send(response);
          }).catch((error) => {
              throw new Error(error);
          });
      }
  }

    listQtdeTotalProdutosFunc(id_func, callback) {
      return new Promise(async (resolve, reject) => {
        // const query = data_concluido ? `pedidos.data = ?` : `pedidos.data <= ?`
        // const data_formatada = data_concluido ? data_concluido : new Date();
    
      
        try {
          db.query(`SELECT 
          SUM(pedidos_func.quantidade) AS quantidade_total
          FROM pedidos_func
          JOIN produtos_func ON pedidos_func.id_produto_func = produtos_func.id
          WHERE 
          pedidos_func.id_func = ? AND 
          pedidos_func.concluido = false`,
            [id_func],
            (error, pedidos) => {
              if (error) return reject(new Error(error));
              return resolve(pedidos);
            }
          );
        } catch (error) {
          return reject(new Error(error));
        }
      });
  }

    /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
    concluirPedido() {
      return function (req, res) {
          const data = req.body;
          
          funcionario.concluirPedido(data).then((response) => {
              if (response.error) res.status(400).send(response);
              else return res.status(200).send(response);
          }).catch((error) => {
              throw new Error(error);
          });
      }
    }

    listFuncDataSocket(id_func) {
      return new Promise(async (resolve, reject) => {  
  
  
        db.query(
          `SELECT * FROM funcionarios WHERE funcionarios.id = ?`,
          [id_func],
          async (error, funcionario) => {
            if (error) return reject(new Error(error));
            return resolve(funcionario);
          }
        );
      });
    }

}

module.exports = funcionarioController;
