const {
    decrypt,
    encrypt,
    generateHash,
    checkHash,
  } = require("../../config/auth/cryptography");
  const { genereteToken } = require("../../config/auth/jwt");
  const { encryptDonor, decryptDonor } = require("../../util/cryptDonor");
  const db = require("../database/db");
  const nodemailer = require("nodemailer");
  const crypto = require("crypto");
  const { response } = require("express");
  const { getMaxListeners } = require("../database/db");
  const path = require("path");
  
  require("dotenv").config({ path: __dirname + "/./../../../.env" });
  
  class descontosRepository {
  
    constructor() {
      this.db = db;
    }
  

  /**
   * Registra um pedido se não houver no banco, caso haja no banco só incrementa a quantidade
   * @param {*} data dados do pedido
   * @returns mensagem referente ao resultado obtido
   */
  register(dataArray, id_user) {
    return new Promise(async (resolve, reject) => {
        console.log(dataArray);
        
        try {
          for (let desconto of dataArray) {
              try {
                  const descontoResults = await new Promise((resolve, reject) => {
                      this.db.query(
                          `SELECT * FROM descontos WHERE id_user = ? AND id_produto = ?`,
                          [id_user, desconto.id_produto],
                          (error, results) => {
                              if (error) {
                                  console.error("Erro ao buscar desconto existente:", error);
                                  return reject(error);
                              }
                              resolve(results);
                          }
                      );
                  });
  
                  if (descontoResults.length > 0) {
                      // Desconto já existe, fazer update
                      await new Promise((resolve, reject) => {
                        this.db.query(
                              `UPDATE descontos SET desconto = ?, operacao = ? WHERE id_user = ? AND id_produto = ?`,
                              [desconto.desconto, desconto.operacao, id_user, desconto.id_produto],
                              (error) => {
                                  if (error) {
                                      console.error("Erro ao atualizar desconto:", error);
                                      return reject(error);
                                  }
                                  resolve();
                              }
                          );
                      });
                  } else {
                      // Desconto não existe, fazer insert
                      await new Promise((resolve, reject) => {
                        this.db.query(
                              `INSERT INTO descontos (desconto, operacao, id_user, id_produto) VALUES (?, ?, ?, ?)`,
                              [desconto.desconto, desconto.operacao, id_user, desconto.id_produto],
                              (error) => {
                                  if (error) {
                                      console.error("Erro ao inserir desconto:", error);
                                      return reject(error);
                                  }
                                  resolve();
                              }
                          );
                      });
                  }
              } catch (error) {
                  console.error('Erro ao processar desconto:', error);
                  return { error: "STATIC_ERRO_VALIDACAO_SQL", code: 203 };
              }
          }
  
          return resolve({ success: "Descontos processados com sucesso!", code: 200 });
      } catch (error) {
          console.error('Erro ao registrar descontos:', error);
          return { error: "STATIC_ERRO_VALIDACAO_SQL", code: 203 };
      }
  })
  }

      

  
    /**
     * Lista os dados de um desconto
     * @returns lista com todos os dados de um desconto
     */
    list(id) {
      return new Promise(async (resolve, reject) => {
        var query = id
          ? `WHERE id = ?`
          :``
  
        try {
          this.db.query(`SELECT * FROM descontos ${query}`, [id ? id : ""], 
          async (error, produtos) => {
            
            if (error) { 
              
              console.log(error);
              return reject(new Error(error));
            }
  
            return resolve(produtos);
          });
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }

  /**
   * Lista os descontos de um usuario
   * @returns lista com todos os descontos de um usuario
   */
      listDescontosByUser(id_user) {
        return new Promise(async (resolve, reject) => {

    
          try {
            this.db.query(`SELECT descontos.id, descontos.desconto, descontos.id_user, descontos.operacao, descontos.id_produto, produtos.nome
                        FROM descontos, user, produtos
                        WHERE
                        descontos.id_produto = produtos.id AND
                        descontos.id_user = user.id AND
                        descontos.id_user = ?`, [id_user ? id_user : ""], 
            async (error, produtos) => {
              
              if (error) { 
                
                console.log(error);
                return reject(new Error(error));
              }
    
              return resolve(produtos);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }

    
  
    /**
     * Deleta uma unidade cadastrada
     * @param {*} id Id da unidade a ser excluída
     * @returns mensagem referente ao resultado obtido
     */
    delete(id) {
      return new Promise((resolve, reject) => {
  
        this.db.query('DELETE FROM descontos WHERE id = ?', [id],
        (error, response) => {
            if (error) {
                return resolve({ error: "Esta descontos possui vinculos!", code: 203 });
            }
            else {
                return resolve(true);
            }
        });
      });
    }

       /**
     * Deleta uma unidade cadastrada
     * @param {*} id Id da unidade a ser excluída
     * @returns mensagem referente ao resultado obtido
     */
       deletePedidoByUserId(id_user) {
        return new Promise((resolve, reject) => {
    
          this.db.query('DELETE FROM pedidos WHERE id_user = ?', [id_user],
          (error, response) => {
              if (error) {
                  return resolve({ error: "Este pedidos possui vinculos!", code: 203 });
              }
              else {
                  return resolve(true);
              }
          });
        });
      }
    
  
    /**
     * Atualiza os dados de um produto
     * @param {*} data dados atualizados do produto
     * @returns mensagem referente ao resultado obtido
     */
    update(data, id_user) {
      return new Promise(async (resolve, reject) => {

        const results = [];

        try {
          for (let i = 0; i < data.length; i++) {
            const descontos = data[i];
    
            // Executa a query para cada item
            const response = await new Promise((resolve, reject) => {
              this.db.query(
                "UPDATE descontos SET desconto = ?, operacao = ? WHERE id_produto = ? AND id_user = ?",
                [
                  descontos.desconto,
                  descontos.operacao,
                  descontos.id_produto,
                  id_user
                ],
                (error, response) => {
                  if (error) return reject(new Error(error));
                  resolve({ success: `Desconto para o produto ${data.id_produto} atualizado com sucesso!` });
                }
              );
            });
    
            results.push(response);
          }
    
          return resolve({ sucess: `Descontos atualizados`, code: 200 });  // Retorna todos os resultados das atualizações
        } catch (err) {
          return reject(err);  // Em caso de erro, rejeita a Promise
        }
        


      })

  }
}
  
  module.exports = descontosRepository;
  