const {
  decrypt,
  encrypt,
  generateHash,
  checkHash,
} = require("../../config/auth/cryptography");
const { genereteToken } = require("../../config/auth/jwt");
const { encryptDonor, decryptDonor } = require("../../util/cryptDonor");
const db = require("../database/db");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const { response } = require("express");
const fs = require('fs');
const { getMaxListeners } = require("../database/db");
const { sendMessage, sendMedia } = require("../../util/gzappy")
const path = require("path");
const { trocarAno, anoMesDia } = require("../../util/dateFormat");
const { getLastId, saveLastId } = require("../../util/geradorId");
const { default: axios } = require("axios");
const { getSignedUrl } = require("@aws-sdk/s3-request-presigner");
const { S3Client, PutObjectCommand, GetObjectCommand, PutObjectAclCommand } = require('@aws-sdk/client-s3');
require("dotenv").config({ path: __dirname + "/./../../../.env" });

class pedidosRepository {

  constructor() {
    this.db = db;
    this.wsdlUrl = process.env.SOAP_WSDL_URL;
    this.soapAction = process.env.SOAP_ACTION;
    this.s3Client = new S3Client({
      region: process.env.S3_REGION,
      credentials: {
        accessKeyId: process.env.AWS_ACCESS_KEY,
        secretAccessKey: process.env.AWS_SECRET_KEY,
      },
    });
  }



  async makePublic() {
    const params = {
        Bucket: process.env.S3_BUCKET,
        Key: "pedidos1",
        ACL: "public-read", 
    };

    try {
        await this.s3Client.send(new PutObjectAclCommand(params));
        console.log("Objeto tornado público com sucesso!");
    } catch (err) {
        console.error("Erro ao tornar o objeto público:", err);
    }
};


  /**
   * Registra um pedido se não houver no banco, caso haja no banco só incrementa a quantidade
   * @param {*} data dados do pedido
   * @returns mensagem referente ao resultado obtido
   */
  register(data, id_user) {
    return new Promise(async (resolve, reject) => {
      console.log(data);

      let insert;
      let pedidoId;


      try {
        // Verificar se o pedido já existe
        const pedidoResults = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT * FROM pedidos WHERE id_user = ? AND id_produto = ? AND concluido = false`,
            [data.id_user, data.id_produto],
            (error, results) => {
              if (error) {
                console.error("Error fetching existing order:", error);
                return reject(error);
              }
              resolve(results);
            }
          );
        });


        console.log("Existing order: ", pedidoResults);

        let qtde = parseInt(data.quantidade) // Inicializa a quantidade com o valor da nova inserção

        if (pedidoResults.length > 0) {
          // Se o pedido já existir, atualiza a quantidade
          const existingOrder = pedidoResults[0];
          qtde += existingOrder.quantidade; // Soma a quantidade existente com a nova

          console.log("Updated quantity:", qtde);

          await new Promise((resolve, reject) => {
            this.db.query(
              `UPDATE pedidos SET quantidade = ? WHERE id_user = ? AND id_produto = ? AND concluido = false`,
              [qtde, data.id_user, data.id_produto],
              (error) => {
                if (error) {
                  console.error("Error updating order:", error);
                  return reject(error);
                }
                resolve();
              }
            );
          });
        } else {
          // Se o pedido não existir, insere o novo pedido
          console.log("Inserting new order");

          await new Promise((resolve, reject) => {
            this.db.query(
              `INSERT INTO pedidos (id_user, id_produto, quantidade) VALUES (?, ?, ?)`,
              [data.id_user, data.id_produto, data.quantidade],
              (error) => {
                if (error) {
                  console.error("Error inserting order:", error);
                  return reject(error);
                }
                resolve();
              }
            );
          });
        }

        return resolve({ success: "Pedido adicionado com sucesso!", code: 200 });
      } catch (error) {
        console.error('Erro ao registrar pedido:', error);
        resolve({ error: "STATIC_ERRO_VALIDACAO_SQL", code: 203 });
      }
    });
  }


  /**
   * Lista os dados de uma unidade
   * @returns lista com todos os dados de uma unidade
   */
  list(id_user, data, hora) {
    return new Promise(async (resolve, reject) => {

      let dataFormatada = anoMesDia(data)
      console.log(dataFormatada);


      try {
        this.db.query(`SELECT pedidos.id, pedidos.id_user, pedidos.id_produto, produtos.nome, produtos.image, produtos.descricao, pedidos.quantidade, pedidos.concluido, date_format(pedidos.data, "%d/%m/%Y") AS data, pedidos.hora 
                      FROM pedidos, produtos
                      WHERE 
                      pedidos.id_produto = produtos.id AND
                      pedidos.id_user = ? AND
                      pedidos.data = ? AND
                      pedidos.hora = ? AND
                      pedidos.concluido = true`, [id_user, dataFormatada, hora],
          async (error, pedidos) => {

            if (error) {

              console.log(error);
              return reject(new Error(error));
            }

            return resolve(pedidos);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * Lista os dados de uma unidade
   * @returns lista com todos os dados de uma unidade
   */
  listPedidosConcluidos(id, id_user) {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(`SELECT produtos.nome, produtos.id as id_produto, pedidos.quantidade, produtos.image, produtos.descricao, user.id AS id_user, pedidos.id as id_pedido, DATE_FORMAT(pedidos.data, '%d/%m/%Y') AS data
                            FROM produtos, pedidos, user
                            WHERE 
                            user.id = ? AND
                            pedidos.concluido = true AND
                            produtos.id = pedidos.id_produto AND
                            pedidos.id_user = user.id
                            ORDER BY pedidos.data DESC`,
          [id_user ? id_user : ""],
          async (error, pedidos) => {

            if (error) {

              console.log(error);
              return reject(new Error(error));
            }

            return resolve(pedidos);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
  * Lista os dados de uma unidade
  * @returns lista com todos os dados de uma unidade
  */
  listPedidosPendentes(id, id_user) {
    return new Promise(async (resolve, reject) => {

      if (!id_user) return resolve({ error: `ID do usuario obrigatorio`, code: 203 })

      try {

        let qtdeMinimaSuino;
        let qtdeMinimaBovino;

        const qtdeTotalProdutos = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT SUM(pedidos.quantidade) AS qtde_total FROM pedidos WHERE id_user = ? AND concluido = false`,
            [id_user],
            (error, results) => {
              if (error) {
                console.error("Error fetching existing order:", error);
                return reject(error);
              }
              resolve(results[0].qtde_total);

            }
          );
        });


        const qtdeMinima = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT user.qtde_minima_suino, qtde_minima_bovino FROM user WHERE user.id = ?`,
            [id_user],
            (error, results) => {
              if (error) {
                console.error("Error fetching existing order:", error);
                return reject(error);
              }
              resolve(results[0]);
              qtdeMinimaBovino = results[0].qtde_minima_bovino
              qtdeMinimaSuino = results[0].qtde_minima_suino

            }
          );
        });

        this.db.query(`SELECT produtos.nome, produtos.id as id_produto, pedidos.quantidade, produtos.image, produtos.descricao, user.id AS id_user, pedidos.id as id_pedido, DATE_FORMAT(pedidos.data, '%d/%m/%Y') AS data,
                      produtos.categoria
                      FROM produtos, pedidos, user
                      WHERE 
                      user.id = ? AND
                      pedidos.concluido = false AND
                      produtos.id = pedidos.id_produto AND
                      pedidos.id_user = user.id
                      ORDER BY pedidos.data DESC`,
          [id_user ? id_user : ""],
          async (error, pedidos) => {

            if (error) {

              console.error(error);
              return reject(new Error(error));
            }

            if (qtdeMinimaSuino > qtdeTotalProdutos || qtdeMinimaBovino > qtdeTotalProdutos) {
              pedidos = pedidos.map(produto => {
                produto.message = 'true';
                return produto;
              });
            }

            // for (const produto of pedidos) {

            //   // if (produto.categoria === 2) {
            //   //   const url = "https://brasapedidos.s3.us-east-2.amazonaws.com/" + produto.image;
            //   //   produto.image = url;
            //   // }
      
            // }
            return resolve(pedidos);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * Lista os dados de uma unidade
   * @returns lista com todos os dados de uma unidade
   */
  listPedidosFeitos(id_pedido, id_user) {
    return new Promise(async (resolve, reject) => {
      const query = id_pedido ? `pedidos.id = ? AND` : ``


      try {
        this.db.query(`SELECT produtos.nome, produtos.id as id_produto, pedidos.quantidade, produtos.image, produtos.descricao, user.id AS id_user, pedidos.id as id_pedido
                          FROM produtos, pedidos, user
                          WHERE 
                          user.id = ? AND
                          ${query}
                          pedidos.concluido = true AND
                          produtos.id = pedidos.id_produto AND
                          pedidos.id_user = user.id
                          ORDER BY pedidos.data DESC`,
          [id_user, id_pedido ? id_user && id_pedido : ``],

          async (error, pedidos) => {
            if (error) {
              console.log(error);
              return reject(new Error(error));
            }
            return resolve(pedidos);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * Lista a quantidade de todos os produtos em um pedido
   * @returns lista quantidade de todos os produtos em um pedido
   */
  listQtdeTotalProdutos(id_user, callback) {
    this.db.query(
      `SELECT SUM(pedidos.quantidade) AS quantidade_produtos 
        FROM pedidos
        WHERE pedidos.id_user = ? AND pedidos.concluido = false`,
      [id_user],
      (error, result) => {
        if (error) {
          console.log(error);
          return callback({ error: "Erro ao consultar a quantidade total de produtos." });
        }

        // Verifica se algum resultado foi retornado
        if (result.length === 0 || result[0].quantidade_produtos === null) {
          return callback(null, { quantidade_produtos: 0 });
        }

        return callback(null, result[0]);
      }
    );
  }

  listForm(dataInicio, dataFim) {
    return new Promise(async (resolve, reject) => {

      if (!dataInicio || !dataFim) return resolve({ error: `Data não atribuida!`, code: 203 })


      this.db.query(
        `WITH recursive AllDates AS (
            SELECT DATE(?) AS data -- Início do intervalo
            UNION ALL
            SELECT data + INTERVAL 1 DAY
            FROM AllDates
            WHERE data + INTERVAL 1 DAY <= DATE(?)
          )
          SELECT
              DATE_FORMAT(AllDates.data, "%d/%m") AS data,
              COALESCE(SUM(CASE WHEN vendedores.contrato = 'CLT' THEN pedidos.quantidade ELSE 0 END), 0) AS total_clt,
              COALESCE(SUM(CASE WHEN vendedores.contrato = 'PJ' THEN pedidos.quantidade ELSE 0 END), 0) AS total_pj,
              COALESCE(SUM(CASE WHEN user.id_vendedor IS NULL THEN pedidos.quantidade ELSE 0 END), 0) AS total_sem_vinculo,
              SUM(COALESCE(
                SUM(CASE WHEN vendedores.contrato = 'CLT' THEN pedidos.quantidade ELSE 0 END) + 
                SUM(CASE WHEN vendedores.contrato = 'PJ' THEN pedidos.quantidade ELSE 0 END) + 
                SUM(CASE WHEN user.id_vendedor IS NULL THEN pedidos.quantidade ELSE 0 END),
              0)) OVER (ORDER BY AllDates.data) AS total_vendas_acumulado
          FROM 
              AllDates
          LEFT JOIN 
              pedidos ON DATE(pedidos.data) = AllDates.data
          LEFT JOIN 
              user ON pedidos.id_user = user.id
          LEFT JOIN 
              vendedores ON user.id_vendedor = vendedores.id
          WHERE 
              pedidos.concluido = 1 OR pedidos.concluido IS NULL
          GROUP BY 
              AllDates.data
          ORDER BY 
              AllDates.data
            Limit 366`,
        [dataInicio, dataFim],
        async (error, user) => {

          // for (let i = 0; i<user.length; i++) {

          //   user.total_vendas_acumulado += user[i].total_vendas_acumulado

          // }

          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }

  listFormByPeriodo(periodo, id_user) {
    return new Promise(async (resolve, reject) => {

      if (!periodo) return resolve({ error: `periodo obrigatorio!`, code: 203 })
      if (!id_user) return resolve({ error: `id do usuario obrigatorio!`, code: 203 })



      let query, query2, query3, query4 = ``
      switch (periodo) {
        case 'semanal':
          query = `DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) DAY) AS data`
          query2 = `DATE_ADD(data, INTERVAL 1 DAY) <= DATE_ADD(DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) DAY), INTERVAL 6 DAY)`
          query3 = `DAY`
          query4 = `"%d/%m"`

          break;
        case 'mensal':
          query = `DATE_FORMAT(DATE_SUB(CURDATE(), INTERVAL DAY(CURDATE()) - 1 DAY), '%Y-%m-%d') AS data`
          query2 = `DATE_ADD(data, INTERVAL 1 DAY) <= LAST_DAY(CURDATE())`
          query3 = `DAY`
          query4 = `"%d"`
          
          break;
        case 'anual':
          query = `DATE_FORMAT(CURDATE(), '%Y-01-01') AS data`
          query2 = `DATE_ADD(data, INTERVAL 1 DAY) <= DATE_FORMAT(CURDATE(), '%Y-12-31')`
          query3 = `MONTH`
          query4 = `"%d/%m"`

          break;
        default:
          query = ``
          query2 = ``
          query3 = ``
          query4 = ``


      }


      this.db.query(
        `WITH recursive AllDates AS (
            SELECT ${query}
            UNION ALL
            SELECT data + INTERVAL 1 ${query3} 
            FROM AllDates
            WHERE ${query2}
          )
          SELECT
              DATE_FORMAT(AllDates.data, ${query4}) AS data,
               COALESCE(SUM(CASE WHEN pedidos.id_user = ${id_user} THEN pedidos.quantidade ELSE 0 END), 0) AS caixas,
             COALESCE(COUNT(CASE WHEN pedidos.id_user = ${id_user} THEN 1 ELSE NULL END), 0) AS pedidos
          FROM 
              AllDates
          LEFT JOIN 
              pedidos ON DATE(pedidos.data) = AllDates.data
          LEFT JOIN 
              user ON pedidos.id_user = user.id
          WHERE 
              pedidos.concluido = 1 OR pedidos.concluido IS NULL
          GROUP BY 
              AllDates.data
          ORDER BY 
              AllDates.data
            Limit 366`,
        [],
        async (error, user) => {

          if (error) return reject(new Error(error));
          console.log(user);
          const dadosUnicos = [];
          const seen = new Set();
          

          const meses = [
            "janeiro", "fevereiro", "março", "abril", "maio", "junho", 
            "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"
          ];
          const dia_semana = [
            "domingo" ,"segunda", "terça", "quarta", "quinta", "sexta", "sabado", "sexta"
          ]

           user.forEach(async (item) => {

            if (!seen.has(item.data)) {
              seen.add(item.data);

              if (periodo === "anual") {
                const mesIndex = parseInt(item.data.split("/")[1], 10) - 1;
                item.data = meses[mesIndex];  
              }

              if (periodo === "semanal") {
                const dataParts = item.data.split("/"); 
                const dia = dataParts[0];  
                const mes = dataParts[1] - 1; 
              
                const date = new Date(new Date().getFullYear(), mes, dia); // Usa o ano atual
              

                const diaIndex = date.getDay();
              

                item.data = dia_semana[diaIndex];
              }

            }
          dadosUnicos.push(item);
          
        });
        periodo === "anual" ? dadosUnicos.pop() : ``
          return resolve(dadosUnicos)
        }
      );
    });
  }


  /**
   * Deleta uma unidade cadastrada
   * @param {*} id Id da unidade a ser excluída
   * @returns mensagem referente ao resultado obtido
   */
  delete(id) {
    return new Promise((resolve, reject) => {

      this.db.query('DELETE FROM pedidos WHERE id = ?', [id],
        (error, response) => {
          if (error) {
            return resolve({ error: "Esta pedidos possui vinculos!", code: 203 });
          }
          else {
            return resolve(true);
          }
        });
    });
  }

  /**
   * Deleta uma unidade cadastrada
   * @param {*} id Id da unidade a ser excluída
   * @returns mensagem referente ao resultado obtido
   */
  deletePedidoByUserId(id_user) {
    return new Promise((resolve, reject) => {

      this.db.query('DELETE FROM pedidos WHERE id_user = ?', [id_user],
        (error, response) => {
          if (error) {
            return resolve({ error: "Este pedidos possui vinculos!", code: 203 });
          }
          else {
            return resolve(true);
          }
        });
    });
  }

  /**
   * Deleta uma unidade cadastrada
   * @param {*} id Id da unidade a ser excluída
   * @returns mensagem referente ao resultado obtido
   */
  deleteProdutoDoPedido(id_produto, id_user) {
    return new Promise((resolve, reject) => {

      this.db.query('DELETE FROM pedidos WHERE id_produto = ? AND id_user = ? AND concluido = false', [id_produto, id_user],
        (error, response) => {
          if (error) {
            return resolve({ error: "Este pedido possui vinculos!", code: 203 });
          }
          else {
            return resolve(true);
          }
        });
    });
  }


  /**
   * Atualiza os dados de uma unidade
   * @param {*} data dados atualizados da unidade
   * @returns mensagem referente ao resultado obtido
   */
  update(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE pedidos SET nome=?, endereco=?, cep=?, telefone=?, cidade=?, estado=? WHERE id = ?",
        [
          data.nome,
          data.endereco,
          data.cep,
        ],
        (error, response) => {
          if (error) return reject(new Error(error));
          else return resolve({ success: "Pedido atualizado com sucesso!" });
        }
      );
    });
  }

  /**
   * Conclui um pedido de um usuario 
   * @param {*} data dados de um pedido a se concluido
   * @returns mensagem referente ao resultado obtido
   */
  concluirPedido(packet) {
    return new Promise(async (resolve, reject) => {
      try {
        const dateAtual = new Date();
        const horaFormatada = dateAtual.toLocaleTimeString('pt-BR', { hour: '2-digit', minute: '2-digit' });
        const dataFormatada = dateAtual.toISOString().split('T')[0];
        let vendedor;


        const ultra = await new Promise((resolve, reject) => {
          this.db.query(
              `SELECT * FROM ultra`,[],
              (error, ultra) => {
                  if (error) {
                      console.error("Error: ", error);
                      return reject(error);
                  }
                  resolve(ultra[0]);
              }
          );
      });


    
        const pedido = await new Promise((resolve, reject) => {
          this.db.query(
              `SELECT pedidos.id_user, pedidos.id_produto, pedidos.quantidade FROM pedidos WHERE concluido = false AND id_user = ?`, [packet.id_user],
              (error, pedido) => {
                  if (error) {
                      console.error("Error fetching orders: ", error);
                      return reject(error);
                  }
                  resolve(pedido);
              }
          );
      });
      
      const descontos = await Promise.all(
          pedido.map(p =>
              new Promise((resolve, reject) => {
                  this.db.query(
                      `SELECT * FROM descontos WHERE id_produto = ? AND id_user = ?`, [p.id_produto, packet.id_user],
                      (error, desconto) => {
                          if (error) {
                              console.error(`Error fetching discount for product ${p.id_produto}: `, error);
                              return reject(error);
      
                              
                          }
                          resolve(desconto[0] || { percentual: 0 }); // Retorna 0% desconto se não houver registro.
                      }
                  );
              })
          )
      );
      
      if (ultra.ultra == 1) { 
      const produtos = await Promise.all(
          pedido.map(p =>
              new Promise((resolve, reject) => {
                  this.db.query(
                      `SELECT * FROM produtos_ultra WHERE codProduto = ?`, [p.id_produto],
                      (error, result) => {
                          if (error) {
                              console.error(`Error fetching product ${p.id_produto}: `, error);
                              return reject(error);
                          }
                          resolve(result[0]);
                      }
                  );
              })
          )
      );
      
      
      let ultimoId = await getLastId()+1
      
      const itens = produtos.map((produto, index) => {
        const precoOriginal = produto.preco;
        const qtdPedido = pedido[index].quantidade;
        const descontoPercentual = descontos[index].desconto || 0; 
      
        const valorDescontoUnitario = (precoOriginal * descontoPercentual) / 100;
      
        // Preço líquido por unidade após o desconto
        const precoLiquidoUnitario = precoOriginal - valorDescontoUnitario;
      
        // Cálculo do valor total (para a quantidade do pedido)
        const valorTotalOriginal = precoOriginal * qtdPedido;
        const valorDesconto = valorDescontoUnitario * qtdPedido;
        const valorLiquido = precoLiquidoUnitario * qtdPedido;
      
        console.log("VALOR DESCONTO: ", valorDesconto);
        console.log("VALOR LIQUIDO: ", valorLiquido);
      
        console.log("VALOR DESCONTO UNI ", valorDescontoUnitario);
        
        
        return {
            codGrupoNivel1: produto.codGrupoNivel1,
            codGrupoNivel2: produto.codGrupoNivel2,
            codGrupoNivel3: produto.codGrupoNivel3,
            codPedido: ultimoId,
            codProduto: produto.codProduto,
            codProdutoClas: produto.codProdutoClass,
            codTributacaoUF: 0,
            codTributacaoUFDest: 0,
            codUnidadeMedida: 0,
            descUnidadeMedida: "",
            grupoFiscal: produto.grupoFiscal,
            item: index + 1,
            margemST: 0.0,
            perIPI: 0.0,
            percBaseICMSUF: 0.0,
            percBaseICMSUFDest: 0.0,
            percICMSUF: 0.0,
            percICMSUFDest: 0.0,
            pesoEmbalagem: produto.pesoEmbalagem,
            pesoRepesagem: "0,0000",
            preco: precoLiquidoUnitario,
            qtdAtual: 0.0,
            qtdPedido: qtdPedido,
            qtdUnMed: produto.pesoProduto,
            quantidade: qtdPedido,
            unidadeMedida: produto.undMedida,
            valorBaseICMSUF: 0.0,
            valorBaseICMSUFDest: 0.0,
            valorBaseIPI: 0.0,
            valorBaseST: 0.0,
            valorBruto: valorLiquido.toFixed(2),
            valorDesconto: valorDescontoUnitario,
            valorICMSUF: 0.0,
            valorICMSUFDest: 0.0,
            valorIPI: 0.0,
            valorInformado: precoLiquidoUnitario,
            valorLiquido: valorLiquido.toFixed(2),
            valorRepesagem: "0.00",
            valorST: 0.0,
            valorTotal: valorLiquido.toFixed(2),
            vlrAtual: 0.0,
            vlrTotalAtual: 0.0
        };
      });
      
      // Calcula o valor líquido total para o envelope SOAP.
      const valorLiquidoTotal = itens.reduce((acc, item) => acc + parseFloat(item.valorLiquido), 0);
      
      const data = new Date().toISOString().split('T')[0];
      const dataAtual = new Date();
      
      dataAtual.setDate(dataAtual.getDate() + 2);
      
      const dataEntrega = dataAtual.toISOString().split('T')[0];
      
      const soapEnvelope = `<?xml version="1.0" encoding="utf-8"?>
      <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xmlns:xsd="http://www.w3.org/2001/XMLSchema"
      xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
      <soap12:Body>
      <SaveAllOrders xmlns="http://www.ultrasistemas.com.br/">
      <strPedidos>
      [{
      "codCondRec": 4,
      "codFilial": 1,
      "codForma": 2,
      "codOperacao": 100,
      "codTabelaPreco": 1,
      "codVendedor": 10156,
      "cpfCnpj": "372.236.674-78",
      "data": "${data}",
      "dataEntrega": "${dataEntrega}",
      "emailCliente": "",
      "id":  ${ultimoId},
      "itens": ${JSON.stringify(itens)},
      "md5": "${crypto.randomBytes(9).toString('hex').slice(0, 12)}",
      "newCodigoFV": 0,
      "observacao": "PEDIDO TESTE NAO FATURAR",
      "parceiro": 10204,
      "codigo": ${ultimoId},
      "pesoBruto": 0.0,
      "pesoLiquido": 0.0,
      "qtdeVolumes": 0,
      "valorBaseST": 0.0,
      "valorDesconto": "${itens.reduce((acc, item) => acc + parseFloat(item.valorDesconto), 0).toFixed(2)}",
      "valorIPI": 0.0,
      "valorBruto": "0.00",
      "valorST": 0.0,
      "valorLiquido": "0.00"
      }]
      </strPedidos>
      </SaveAllOrders>
      </soap12:Body>
      </soap12:Envelope>`;
      
      
      await saveLastId(ultimoId);
      
      
          try {
            const response = await axios.post(this.wsdlUrl, soapEnvelope, {
              headers: {
                'Content-Type': 'application/soap+xml; charset=utf-8',
                'SOAPAction': this.soapAction, 
              },
            });
        
            console.log('SOAP Response: ', response);
            
            
          } catch (error) {
            if (error.response) {
              // O servidor respondeu com um status HTTP fora do intervalo 2xx
              console.error("Status:", error.response.status);
              // console.error("Headers:", error.response.headers);
              console.error("Dados da resposta:", error.response.data);
            } else if (error.request) {
              // A requisição foi feita, mas nenhuma resposta foi recebida
              // console.error("Nenhuma resposta do servidor. Detalhes da requisição:", error.request);
            } else {
              // Erro na configuração da requisição
              // console.error("Erro ao configurar a requisição:", error.message);
            }
          }

        }



  
        // Busca informações do cliente
        const cliente = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT * FROM user WHERE id = ?`, [packet.id_user],
            (error, results) => {
              if (error) {
                console.error("Erro ao buscar usuário: ", error);
                return reject(error);
              }
              resolve(results[0]);
            }
          );
        });
        
        cliente.id_vendedor !== null ?
           vendedor = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT * FROM vendedores WHERE id = ?`, [cliente.id_vendedor],
              (error, results) => {
                if (error) {
                  console.error("Erro ao buscar usuário: ", error);
                  return reject(error);
                }
                resolve(results[0]);
              }
            );
          })
          : ``
  
        
        const produtosPedidos = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT produtos.nome, produtos.id AS id_produto, pedidos.quantidade, pedidos.aroma, produtos.nome_unidade, produtos.descricao_unidade
             FROM produtos
             INNER JOIN pedidos ON pedidos.id_produto = produtos.id
             WHERE pedidos.id_user = ? AND pedidos.concluido = false`,
            [packet.id_user],
            (error, results) => {
              if (error) {
                console.error("Erro ao buscar produtos do pedido: ", error);
                return reject(error);
              }
              resolve(results);
            }
          );
        });
        
        
        if (produtosPedidos.length == 0) return resolve({ error: `Adicione itens no carrinho`, code: 203 })

        
  
        // Atualiza pedido no banco de dados
        await new Promise((resolve, reject) => {
          this.db.query(
            `UPDATE pedidos SET concluido = ?, data = ?, hora = ?, aroma = ?, observacao = ?
             WHERE id_user = ? AND concluido = false`,
            [packet.concluido, dateAtual, horaFormatada, packet.aroma, packet.observacao, packet.id_user],
            (error) => {
              if (error) {
                console.error("Erro ao atualizar pedido: ", error);
                return reject(error);
              }
              resolve();
            }
          );
        });
        console.log(produtosPedidos);
        
        let mensagemCliente;
cliente.role == "FUNC" ? 
        mensagemCliente = `*Aliança Pedidos - Pedido Realizado!*\n
${produtosPedidos.map((p, index) => `*Produto ${index+1}*
- ${p.nome_unidade}
- ${p.descricao_unidade.toLowerCase()}
- ${p.quantidade} Unidades\n`).join('\n')}
*Observações do Pedido:*
${packet.observacao || 'Nenhuma observação adicionada'}\n
*Para mais pedidos acesse:*\nhttps://aliancapedidos.com.br`
:
mensagemCliente = `*Aliança Pedidos - Pedido Realizado!*\n
${produtosPedidos.map((p, index) => `*Produto ${index+1}*
- ${p.quantidade}x ${p.nome}\n`).join('\n')}
*Observações do Pedido:*
${packet.observacao || 'Nenhuma observação adicionada'}\n
*Para mais pedidos acesse:*\nhttps://aliancapedidos.com.br`

  let sellerMessage

vendedor?.contrato === "RH" ?
        sellerMessage = `${cliente.razao_social || 'Não disponível'}
${cliente.documento}\n
${produtosPedidos.map(p => `${p.id_produto} = ${p.quantidade} Unidades`).join('\n')}
${packet.observacao ? `\n${packet.observacao}` : '\nNenhuma observação adicionada'}`
:
sellerMessage = `${cliente.razao_social || 'Não disponível'}
${cliente.documento}\n
${produtosPedidos.map(p => `${p.id_produto} = ${p.quantidade}x`).join('\n')}
${packet.observacao ? `\n${packet.observacao}` : '\nNenhuma observação adicionada'}`


          const fotosAWS = "alianca"

      cliente.notificacao_whats == true ?
        await sendMedia(mensagemCliente,  "+55"+cliente.celular, fotosAWS)
      : ``

      if (cliente.notificacao_email == true) { 
          const templatePath = path.join(__dirname, '../../view/pedido-concluido.html');
          let templateHtml = fs.readFileSync(templatePath, 'utf-8');

          const data = new Date();
          const dia = String(data.getDate()).padStart(2, '0');
          const mes = String(data.getMonth() + 1).padStart(2, '0'); 
          const ano = data.getFullYear();

          const formatoData = `${dia}/${mes}/${ano}`;

          
          templateHtml = templateHtml
          .replace(/\[Nome do Cliente\]/g, cliente.razao_social)
          .replace(/\[Documento\]/g, cliente.documento)
          .replace(/\[E-mail do Cliente\]/g, cliente.email)
          .replace(/\[Telefone do Cliente\]/g, cliente.celular)
          .replace(/\[Data do Pedido\]/g, formatoData)

      // Substituir a lista de produtos dinamicamente
      
    
      let produtosHtml = produtosPedidos.map(produto =>
          `<tr><td>${produto.nome}</td><td>${produto.quantidade}</td></tr>`
      ).join('');

      templateHtml = templateHtml.replace(/<tbody>[\s\S]*<\/tbody>/, `<tbody>${produtosHtml}</tbody>`);
          
                
          let transporter = nodemailer.createTransport({
            host: 'brasapedidos.com.br',
            port: 465,
            secure: true,
            auth: {
              user: process.env.EMAIL_SENDER,
              pass: process.env.EMAIL_PASSWORD,
            },
          });

          let mailOptions = {
            from: `suporte@brasapedidos.com.br`,
            to: cliente.email,
            subject: `Pedido Concluido`,
            html: templateHtml,
          };

          transporter.sendMail(mailOptions, (error, info) => {
            if (error) {
              console.error("Error sending email: ", error);
              return reject(error);
            }
            console.log('Email sent: %s', info.messageId);
            resolve({ success: "Email enviado com sucesso!", code: 200 });
          });
      }
      

        cliente.id_vendedor ?
          await sendMessage(sellerMessage, "+55"+vendedor.celular) 
          
          : 
          await sendMessage(sellerMessage, "+55"+ultra.faturamento)


          //19 99687-2202
          // await sendMessage(sellerMessage, "+5519995201285")
 
        return resolve({ success: "Pedido concluído!", code: 200 });
      } catch (error) {
        console.error("Erro ao concluir pedido: ", error);
        return reject(new Error("Erro ao concluir pedido."));
      }
    });
  }
  

  /**
* Atualiza os dados de uma unidade
* @param {*} data dados atualizados da unidade
* @returns mensagem referente ao resultado obtido 
*/
  updateQuantidade(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE pedidos SET quantidade = ? WHERE id_user = ? AND id_produto = ? AND concluido = false",
        [
          data.quantidade,
          data.id_user,
          data.id_produto,
        ],
        (error, response) => {
          if (error) return reject(new Error(error));
          else return resolve({ success: "Quantidade atualizada com sucesso!" });
        }
      );
    });
  }
}

module.exports = pedidosRepository;
