const {
    decrypt,
    encrypt,
    generateHash,
    checkHash,
  } = require("../../config/auth/cryptography");
  const { genereteToken } = require("../../config/auth/jwt");
  const { encryptDonor, decryptDonor } = require("../../util/cryptDonor");
  const db = require("../database/db");
  const nodemailer = require("nodemailer");
  const crypto = require("crypto");
  const { response } = require("express");
  const { getMaxListeners } = require("../database/db");
  const path = require("path");
  
  require("dotenv").config({ path: __dirname + "/./../../../.env" });
  
  class precosRepository {
  
    constructor() {
      this.db = db;
    }


       register(data, id_user) {
            return new Promise(async (resolve, reject) => {
                console.log(data);

                const queries = data.map((preco) => {
                  return new Promise((innerResolve, innerReject) => {
                      this.db.query(
                          `INSERT INTO precos (id_produto, tabela, valor) VALUES (?,?,?)`,
                          [preco.id_produto, preco.tabela, preco.valor],
                          (error) => {
                              if (error) {
                                  console.error("Error inserting order:", error);
                                  return innerReject(error);
                              }
                              innerResolve();
                          }
                      );
                  });
              });

                return resolve({ success: "preco cadastrado com sucesso!", code: 200 });
            });
        }


    /**
     * Lista os dados de um log
     * @returns lista com todos os dados de um log
     */
    list(id) {
      return new Promise(async (resolve, reject) => {
        var query = id
          ? `WHERE id = ?`
          :``
  
        try {
          this.db.query(`SELECT * FROM precos ${query}`, [id ? id : ""], 
          async (error, produtos) => {
            
            if (error) { 
              console.log(error);
              return reject(new Error(error));
            }
  
            return resolve(produtos);
          });
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }

    listPrecoPorUserId(id_user) {
      return new Promise(async (resolve, reject) => {

  
        try {

          const user = new Promise(async (resolve, reject) => {
            this.db.query(`SELECT * FROM user WHERE id = ?`, [id_user], 
            async (error, user) => {
            
            if (error) { 
              console.log(error);
              return reject(new Error(error));
            }
            resolve(user)
            })
          })
        

          const precos = new Promise(async (resolve, reject) => {
            this.db.query(`SELECT * FROM precos WHERE tabela like ? `, [user.tipo], 
            async (error, precos) => {
            
            if (error) { 
              console.log(error);
              return reject(new Error(error));
            }
            resolve(precos)
          });
        });
  
            return resolve(precos);
        
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }

  /**
   * Lista todos os logs 
   * @returns lista com todos os logs 
   */
      listPrecosPorProduto(id_produto) {
        return new Promise(async (resolve, reject) => {
        
          try {
            this.db.query(`SELECT * FROM precos WHERE id_produto = ?`
            ,[id_produto ? id_produto : ``], 
            async (error, precos) => {
              if (error) { 
                console.error(error);
                return reject(new Error(error));
              }


            return resolve(precos);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }

      listTabelaPrecoNome() {
        return new Promise(async (resolve, reject) => {
        
          try {
            this.db.query(`SELECT precos.tabela FROM precos`
            ,[], 
            async (error, precos) => {
              if (error) { 
                console.error(error);
                return reject(new Error(error));
              }


            return resolve(precos);
            });
          } catch (error) {
            console.error(error);
            return reject(new Error(error));
          }
        });
      }

      listProdutosComPrecos() {
        return new Promise((resolve, reject) => {
            try {
                const query = `
                    SELECT 
                        p.id, p.nome, p.categoria, p.linha, p.descricao, p.visivel, 
                        p.exclusivo, p.excluido, p.image, p.ordem,
                        pr.tabela, pr.valor, pr.id AS id_preco
                    FROM produtos p
                    LEFT JOIN precos pr ON p.id = pr.id_produto
                    WHERE p.excluido != 1 
                    ORDER BY p.id, pr.tabela;
                `;
    
                this.db.query(query, [], (error, results) => {
                    if (error) {
                        console.error(error);
                        return reject(new Error(error));
                    }
    
                    const produtosMap = {};
    
                    results.forEach(row => {
                        const { id, nome, descricao, image, linha, categoria, visivel, excluido, exclusivo, ordem, tabela, valor, id_preco } = row;
    
                        if (!produtosMap[id]) {
                            produtosMap[id] = {
                                id,
                                nome,
                                descricao,
                                image,
                                categoria,
                                visivel,
                                excluido,
                                exclusivo,
                                linha,
                                ordem,
                                tabelas: []
                            };
                        }
    
                        if (tabela && valor !== null) {
                            produtosMap[id].tabelas.push({
                                tabela,
                                preco: valor,
                                id: id_preco
                            });
                        }
                    });
                    return resolve(Object.values(produtosMap));
                });
            } catch (error) {
                reject(new Error(error));
            }
        });
    }
    


    
    listPrecosByUserTipo(id_user) {
      return new Promise(async (resolve, reject) => {
        try {
          // Buscar dados do usuário
          const user = await new Promise((resolve, reject) => {
            this.db.query(`SELECT * FROM user WHERE id = ?`, [id_user], (error, user) => {
              if (error) {
                console.error(error);
                return reject(new Error(error));
              }
              resolve(user[0]);
            });
          });
    
          // Buscar preços dos produtos de acordo com a tabela do usuário
          const query = `
            SELECT
            p.id,
            p.linha,
            p.nome, 
            precos.valor
            FROM produtos p
            JOIN precos ON precos.id_produto = p.id
            WHERE precos.tabela = ?;
          `;
    
          const tabelaPreco = await new Promise((resolve, reject) => {
            this.db.query(query, [user.tipo], (error, results) => {
              if (error) {
                console.error(error);
                return reject(new Error(error));
              }
              resolve(results);
            });
          });
    
          // Buscar descontos do usuário
          const descontos = await new Promise((resolve, reject) => {
            this.db.query(`SELECT * FROM descontos WHERE id_user = ?`, [id_user], (error, results) => {
              if (error) {
                console.error(error);
                return reject(new Error(error));
              }
              resolve(results);
            });
          });
    
          // Criar um mapa de descontos por id_produto para facilitar o acesso
          const descontoMap = {};
          descontos.forEach(desc => {
            descontoMap[desc.id_produto] = { valor: desc.desconto, operacao: desc.operacao };
          });
    
          // Aplicar os descontos aos produtos e formatar os dados
          const formattedData = tabelaPreco.reduce((acc, item) => {
            let linhaIndex = acc.findIndex(l => l.linha === item.linha);
            
            if (linhaIndex === -1) {
              acc.push({ linha: item.linha, produtos: [] });
              linhaIndex = acc.length - 1;
            }
    
            let precoFinal = item.valor;
    
            // Se houver desconto para este produto, aplicamos
            if (descontoMap[item.id]) {
              const { valor, operacao } = descontoMap[item.id];
    
              if (operacao === "+") {
                precoFinal += valor;
              } else if (operacao === "-") {
                precoFinal -= valor;
              } else if (operacao === "%") {
                precoFinal -= (precoFinal * valor) / 100;
              }
            }
    
            acc[linhaIndex].produtos.push({
              id: item.id,
              nome: item.nome,
              preco: precoFinal
            });
    
            return acc;
          }, []);
    
          return resolve(formattedData);
        } catch (error) {
          reject(new Error(error));
        }
      });
    }
    
  
    
    listTodosProdutosPorTabela() {
      return new Promise((resolve, reject) => {
          try {
              const query = `
                  SELECT p.id, p.nome, p.categoria,
                         pr.tabela, pr.valor, pr.id AS id_preco
                  FROM produtos p
                  INNER JOIN precos pr ON p.id = pr.id_produto
                  WHERE p.excluido != 1 
                  ORDER BY pr.tabela, p.id;
              `;
  
              this.db.query(query, [], (error, results) => {
                  if (error) {
                      console.error(error);
                      return reject(new Error(error));
                  }
  
                  const tabelasMap = {};
  
                  results.forEach(row => {
                      const { id, nome, tabela, categoria, valor, id_preco } = row;
  
                      if (!tabelasMap[tabela]) {
                          tabelasMap[tabela] = {
                              tabela: tabela,
                              produtos: []
                          };
                      }
  
                      tabelasMap[tabela].produtos.push({
                          id: id,
                          nome: nome,
                          categoria: categoria,
                          preco: valor,
                          id_preco: id_preco
                      });
                  });
  
                  return resolve(Object.values(tabelasMap));
              });
          } catch (error) {
            console.error(error);
            
              reject(new Error(error));
          }
      });
  }
  
      

    /**
     * Deleta uma unidade cadastrada
     * @param {*} id Id da unidade a ser excluída
     * @returns mensagem referente ao resultado obtido
     */
    delete(id) {
      return new Promise((resolve, reject) => {
  
        this.db.query('DELETE FROM precos WHERE id = ?', [id],
        (error, response) => {
            if (error) {
                return resolve({ error: "Esta logs possui vinculos!", code: 203 });
            }
            else {
                return resolve(true);
            }
        });
      });
    }

    updatePreco(data) {
      return new Promise((resolve, reject) => {
        const queries = [];
        
        data.forEach(tabela => {
            tabela.produtos.forEach(produto => {
                const queryPromise = new Promise((res, rej) => {
                    this.db.query(
                        'UPDATE precos SET valor = ? WHERE id = ?',
                        [produto.preco, produto.id_preco],
                        (error, response) => {
                            if (error) {
                                return res({ error: `Erro ao atualizar preço do produto ${produto.nome}`, id_preco: produto.id_preco, code: 203 });
                            }
                            return res({ success: `Preço do produto ${produto.nome} atualizado com sucesso!`, id_preco: produto.id_preco, code: 200 });
                        }
                    );
                });
                queries.push(queryPromise);
            });
        });
        
        Promise.all(queries)
            .then(results => resolve({ success: "Preços atualizados com sucesso!", code: 200 }))
            .catch(err => reject({ error: "Erro ao atualizar preços em lote", code: 500 }));
    });
}

       /**
     * Deleta uma unidade cadastrada
     * @param {*} id Id da unidade a ser excluída
     * @returns mensagem referente ao resultado obtido
     */
       deleteTabela(id_tabela) {
        return new Promise((resolve, reject) => {
    
          this.db.query('DELETE FROM preco WHERE id= ?', [id_tabela],
          (error, response) => {
              if (error) {
                  return resolve({ error: "Este pedidos possui vinculos!", code: 203 });
              }
              else {
                  return resolve(true);
              }
          });
        });
      }

    /**
     * Deleta uma unidade cadastrada
     * @param {*} id Id da unidade a ser excluída
     * @returns mensagem referente ao resultado obtido
     */
    updateTabelaPreco(data) {
    return new Promise((resolve, reject) => {

      this.db.query('UPDATE precos SET tabela = ?, valor = ? WHERE id=?', [data.tabela, data.valor, data.id],
      (error, response) => {
          if (error) return resolve({ error: "Este pedidos possui vinculos!", code: 203 });
          
          return resolve({ error: "Tabela de preço atualizada com sucesso!", code: 200 });
          
      });
    });
  }

    
  
}
  
  module.exports = precosRepository;
  